@@ -85,7 +85,7 @@ pub fn open(header_line: &str) -> Result<Parser> {
 }
 
 impl Parser {
-    pub fn parse<'a, I: Iterator<Item = io::Result<String>> + 'a>(
+    pub fn parse<'a, I: Iterator<Item = io::Result<String>> + Send + 'a>(
         mut self,
         lines: I,
     ) -> Asciicast<'a> {
