@@ -274,7 +274,7 @@ pub struct Record {
     )]
     pub overwrite: bool,
 
-    /// Set a descriptive title that will be stored in the recording metadata. This title may be displayed by players and is useful for organizing and identifying recordings. For example: --title "Installing Podman on Ubuntu".
+    /// Set a title that will be stored in the recording metadata. This title may be displayed by players and is useful for organizing and identifying recordings. For example: --title "Installing Podman on Ubuntu".
     #[arg(short, long, help = "Title of the recording", long_help)]
     pub title: Option<String>,
 
@@ -320,7 +320,7 @@ pub struct Record {
 
 #[derive(Debug, Args)]
 pub struct Play {
-    /// The path to an asciicast file or HTTP(S) URL to play back. HTTP(S) URLs allow playing recordings directly from the web without need for manual downloading. Supported formats include asciicast v1, v2, and v3.
+    /// The path to an asciicast file or HTTP(S) URL to play back. Can be a local file path, HTTP(S) URL for remote files, or '-' to read from standard input. Remote URLs allow playing recordings directly from the web without need for manual downloading. Supported formats include asciicast v1, v2, and v3.
     pub file: String,
 
     /// Control the playback speed as a multiplier of the original timing. Values greater than 1.0 make playback faster, while values less than 1.0 make it slower. For example, --speed 2.0 plays at double speed, while --speed 0.5 plays at half speed. The default is 1.0 (original speed). Can also be set via the config file option playback.speed.
@@ -398,6 +398,27 @@ pub struct Stream {
     #[arg(short, long, help = "Title of the session", long_help)]
     pub title: Option<String>,
 
+    /// Set a description for the session. This description is displayed on the stream page (applies only to remote streaming with --remote) and can include formatting, links, and code blocks. Useful for providing context, instructions, or documentation for viewers.
+    #[arg(
+        long,
+        help = "Description of the session (Markdown supported)",
+        long_help
+    )]
+    pub description: Option<String>,
+
+    /// Set the visibility level for the stream (applies only to remote streaming with --remote). Public streams appear in listings and on your profile page. Unlisted streams are accessible via a direct URL but don't appear in listings. Private streams are only accessible to the owner.
+    #[arg(long, value_enum, help = "Visibility level", long_help)]
+    pub visibility: Option<Visibility>,
+
+    /// Specify the URL of a live audio stream (e.g., Icecast MP3/OGG) to synchronize with the terminal stream (applies only to remote streaming with --remote). When set, viewers can listen to audio commentary while watching the terminal. The audio URL is stored in the stream metadata and used by the player for synchronized playback. For example: --audio-url https://icecast.example.com/live.mp3
+    #[arg(
+        long,
+        value_name = "URL",
+        help = "Audio stream URL for synchronized playback",
+        long_help
+    )]
+    pub audio_url: Option<String>,
+
     /// Stream in headless mode without using the terminal for input/output. This is useful for automated or scripted streaming where you don't want asciinema to interfere with the current terminal session. The streamed command will still execute normally and be visible to viewers, but won't be displayed in your local terminal. Headless mode is enabled automatically when running in an environment where a terminal is not available.
     #[arg(
         long,
@@ -423,6 +444,14 @@ pub struct Stream {
     pub server_url: Option<String>,
 }
 
+/// Visibility level for uploads and streams
+#[derive(Debug, Clone, Copy, clap::ValueEnum)]
+pub enum Visibility {
+    Public,
+    Unlisted,
+    Private,
+}
+
 #[derive(Debug, Args)]
 #[clap(group(ArgGroup::new("mode").args(&["output_file", "stream_local", "stream_remote"]).multiple(true).required(true)))]
 pub struct Session {
@@ -490,10 +519,31 @@ pub struct Session {
     )]
     pub overwrite: bool,
 
-    /// Set a descriptive title for the session that will be stored in the recording metadata and displayed to stream viewers (when doing remote streaming with --remote). For example: --title "Installing Podman on Ubuntu". If the server has stream recording enabled then the title will be included in the recording file created on the server side.
+    /// Set a title for the session that will be stored in the recording metadata and displayed to stream viewers (when doing remote streaming with --remote). For example: --title "Installing Podman on Ubuntu". If the server has stream recording enabled then the title will be included in the recording file created on the server side.
     #[arg(short, long, help = "Title of the session", long_help)]
     pub title: Option<String>,
 
+    /// Set a description for the session. This description is displayed on the stream page (applies only to remote streaming with --stream-remote) and can include formatting, links, and code blocks. Useful for providing context, instructions, or documentation for viewers.
+    #[arg(
+        long,
+        help = "Description of the session (Markdown supported)",
+        long_help
+    )]
+    pub description: Option<String>,
+
+    /// Set the visibility level for the stream (applies only to remote streaming with --stream-remote). Public streams appear in listings and on your profile page. Unlisted streams are accessible via a direct URL but don't appear in listings. Private streams are only accessible to the owner.
+    #[arg(long, value_enum, help = "Stream visibility level", long_help)]
+    pub visibility: Option<Visibility>,
+
+    /// Specify the URL of a live audio stream (e.g., Icecast MP3/OGG) to synchronize with the terminal stream (applies only to remote streaming with --stream-remote). When set, viewers can listen to audio commentary while watching the terminal. The audio URL is stored in the stream metadata and used by the player for synchronized playback. For example: --audio-url https://icecast.example.com/live.mp3
+    #[arg(
+        long,
+        value_name = "URL",
+        help = "Audio stream URL for synchronized playback",
+        long_help
+    )]
+    pub audio_url: Option<String>,
+
     /// Limit the maximum idle time recorded between terminal events to the specified number of seconds. Long pauses (such as when you step away from the terminal) will be capped at this duration in the recording, making playback more watchable. For example, --idle-time-limit 2.0 will ensure no pause longer than 2 seconds appears in the recording. Only applies when --output-file is specified. Note that this option doesn't alter the original (captured) timing information and instead, it embeds the idle time limit value in the metadata, which is interpreted by session players at playback time. This allows tweaking of the limit after recording. Can also be set via the config file option session.idle_time_limit.
     #[arg(
         short,
@@ -572,6 +622,31 @@ pub struct Upload {
     /// The path to the asciicast recording file to upload, in a supported asciicast format (v1, v2, or v3).
     pub file: String,
 
+    /// Set a title for the recording that will be stored in the recording metadata and displayed to viewers. For example: --title "Installing Podman on Ubuntu". This option takes precedence over the "title" field from the recording file itself.
+    #[arg(short, long, help = "Title of the recording", long_help)]
+    pub title: Option<String>,
+
+    /// Set a description for the recording. This description is displayed on the recording page and can include formatting, links, and code blocks. Useful for providing context, instructions, or documentation for viewers.
+    #[arg(
+        long,
+        help = "Description of the recording (Markdown supported)",
+        long_help
+    )]
+    pub description: Option<String>,
+
+    /// Set the visibility level for the recording. Public recordings appear in listings, search results and on your profile page. Unlisted recordings are accessible via a direct URL but don't appear in listings. Private recordings are only accessible to the owner.
+    #[arg(long, value_enum, help = "Recording visibility level", long_help)]
+    pub visibility: Option<Visibility>,
+
+    /// Specify the URL of an audio file (e.g., MP3/OGG) to synchronize with the terminal playback. When set, viewers can listen to audio commentary while watching the terminal. The audio URL is stored in the recording metadata and used by the player for synchronized playback. For example: --audio-url https://example.com/commentary.mp3
+    #[arg(
+        long,
+        value_name = "URL",
+        help = "Audio URL for synchronized playback",
+        long_help
+    )]
+    pub audio_url: Option<String>,
+
     /// Specify a custom asciinema server URL for uploading to self-hosted servers. Use the base server URL (e.g., https://asciinema.example.com). Can also be set via environment variable ASCIINEMA_SERVER_URL or config file option server.url. If no server URL is configured via this option, environment variable, or config file, you will be prompted to choose one (defaulting to asciinema.org), which will be saved as a default.
     #[arg(long, value_name = "URL", help = "asciinema server URL", long_help)]
     pub server_url: Option<String>,
